package asd.td3;

/**
 * Une classe pour l'exercice 5 de la feuille 3.
 * On cacule rcursivement pour tout sites i et j le cot minimum de la
 * connexion du site i au site j, en passant ventuellement par d'autres
 * sites.
 *  2000 Marc Gatano
 */
public class Distance {

    private int[][] c; // c[i][j] = le cout de la connextion directe de i  j
                       // si elle existe, l'infini sinon

    private static int infini = -1;

    /**
     * Construit un objet qui reprsente les connexions directes possibles.
     * Paramtre : le tableau connexion tel que connexion[i][j] = le cot de
     * la connexion directe de i  j. S'il n'y a pas de connexion directe de
     * i  j, connexion[i][j] == infini.
     */
    public Distance(int[][] connexion) {
        c = connexion;
    }

    /**
     * Calcule le cot minimum de la connexion du site i au site j, en passant
     * ventuellement par d'autres sites.
     * Complexit : O(3^n) o n = le nombre de sites.
     */
    public int cout(int i, int j) {
        return cout(i,j,c.length);
    }

    /**
     * Calcule le cot minimum de la connexion du site i au site j, en passant
     * ventuellement par des sites appartenant  [0..k-1].
     * Complexit : O(3^k).
     */
    private int cout(int i, int j, int k) {
        if ( k == 0 ) {
            return c[i][j];
        }
        return min( cout(i,j,k-1), plus( cout(i,k-1,k-1), cout(k-1,j,k-1) ));
    }

    /**
     * Calcule le minimum entre a et b o a et b sont des entiers positifs
     * ou nul ou bien gaux  l'infini.
     */
    private int min(int a, int b) {
        if ( a == infini ) {
            return b;
        }
        if ( b == infini ) {
            return a;
        }
        if ( a < b ) {
            return a;
        }
        return b;
    }

    /**
     * Calcule la somme de a et de b o a et b sont des entiers positifs
     * ou nul ou bien gaux  l'infini.
     */
    private int plus(int a, int b) {
        if ( a == infini ) {
            return infini;
        }
        if ( b == infini ) {
            return infini;
        }
        return a + b;
    }
}
// Pour la complexit :
// La complexit de cout(i,j,k) ne dpend que de k. Soit C(k) cette
// complexit. On a C(k) = C(k-1) + C(k-1) + C(k-1) = 3*C(k-1), soit
// C(k) = O(3^k).
//
